@@ -3,10 +3,11 @@
  *
  * libyahoo2 is available at http://libyahoo2.sourceforge.net/
  *
- * $Revision: 308 $
- * $Date: 2008-04-08 07:09:21 -0700 (Tue, 08 Apr 2008) $
+ * $Revision: 364 $
+ * $Date: 2010-04-13 14:40:19 +0530 (Tue, 13 Apr 2010) $
  * 
  * Copyright (C) 2002-2004, Philip S Tellis <philip.tellis AT gmx.net>
+ * Copyright (C) 2009, Siddhesh Poyarekar <siddhesh.poyarekar@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,6 +49,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <openssl/ssl.h>
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -127,10 +129,12 @@ yahoo_idlabel yahoo_status_codes[] = {
 	{YAHOO_STATUS_IDLE, "Idle"},
 	{YAHOO_STATUS_OFFLINE, "Offline"},
 	{YAHOO_STATUS_CUSTOM, "[Custom]"},
-	{YAHOO_STATUS_NOTIFY, "Notify"},
+	{YPACKET_STATUS_NOTIFY, "Notify"},
 	{0, NULL}
 };
 
+static void ext_yahoo_close(void *fd);
+
 char * yahoo_status_code(enum yahoo_status s)
 {
 	int i;
@@ -142,7 +146,7 @@ char * yahoo_status_code(enum yahoo_status s)
 
 #define YAHOO_DEBUGLOG ext_yahoo_log
 
-int ext_yahoo_log(const char *fmt,...)
+static int ext_yahoo_log(const char *fmt,...)
 {
 	va_list ap;
 
@@ -198,7 +202,7 @@ static char * get_local_addresses()
 	int i;
 	FILE * f;
 	f = popen("netstat -nr", "r");
-	if((int)f < 1)
+	if(!f)
 			goto IP_TEST_2;
 	while( fgets(buff, sizeof(buff), f)  != NULL ) {
 			c = strtok( buff, " " );
@@ -222,7 +226,7 @@ static char * get_local_addresses()
 		gateway[i] = 0;
 
 	f = popen("/sbin/ifconfig -a", "r");
-	if((int)f < 1)
+	if(!f)
 		goto IP_TEST_2;
 
 	while( fgets(buff, sizeof(buff), f) != NULL ) {
@@ -349,7 +353,7 @@ static conf_room * find_conf_room_by_name_and_id(int id, const char * me, const
 	return NULL;
 }
 
-void ext_yahoo_got_conf_invite(int id, const char *me, const char *who, const char *room, const char *msg, YList *members)
+static void ext_yahoo_got_conf_invite(int id, const char *me, const char *who, const char *room, const char *msg, YList *members)
 {
 	conf_room * cr = y_new0(conf_room, 1);
 	cr->room_name = strdup(room);
@@ -368,7 +372,7 @@ void ext_yahoo_got_conf_invite(int id, const char *me, const char *who, const ch
 		print_message(("\t%s", (char *)members->data));
 	
 }
-void ext_yahoo_conf_userdecline(int id, const char *me, const char *who, const char *room, const char *msg)
+static void ext_yahoo_conf_userdecline(int id, const char *me, const char *who, const char *room, const char *msg)
 {
 	YList * l;
 	/* TODO: probably have to use the me arg to find the room */
@@ -388,7 +392,8 @@ void ext_yahoo_conf_userdecline(int id, const char *me, const char *who, const c
 	print_message(("%s declined the invitation from %s to %s\n"
 				"with the message: %s", who, me, room, msg));
 }
-void ext_yahoo_conf_userjoin(int id, const char *me, const char *who, const char *room)
+
+static void ext_yahoo_conf_userjoin(int id, const char *me, const char *who, const char *room)
 {
 	conf_room * cr = find_conf_room_by_name_and_id(id, me, room);
 	if(cr) {
@@ -405,7 +410,8 @@ void ext_yahoo_conf_userjoin(int id, const char *me, const char *who, const char
 	print_message(("%s joined the conference %s [%s]", who, room, me));
 
 }
-void ext_yahoo_conf_userleave(int id, const char *me, const char *who, const char *room)
+
+static void ext_yahoo_conf_userleave(int id, const char *me, const char *who, const char *room)
 {
 	YList * l;
 	conf_room * cr = find_conf_room_by_name_and_id(id, me, room);
@@ -423,7 +429,8 @@ void ext_yahoo_conf_userleave(int id, const char *me, const char *who, const cha
 
 	print_message(("%s left the conference %s [%s]", who, room, me));
 }
-void ext_yahoo_conf_message(int id, const char *me, const char *who, const char *room, const char *msg, int utf8)
+
+static void ext_yahoo_conf_message(int id, const char *me, const char *who, const char *room, const char *msg, int utf8)
 {
 	char * umsg = (char *)msg;
 
@@ -456,12 +463,12 @@ static void print_chat_member(struct yahoo_chat_member *ycm)
 	printf("  Location: %s", ycm->location);
 }
 
-void ext_yahoo_chat_cat_xml(int id, const char *xml) 
+static void ext_yahoo_chat_cat_xml(int id, const char *xml) 
 {
 	print_message(("%s", xml));
 }
 
-void ext_yahoo_chat_join(int id, const char *me, const char *room, const char * topic, YList *members, int fd)
+static void ext_yahoo_chat_join(int id, const char *me, const char *room, const char * topic, YList *members, void *fd)
 {
 	print_message(("You [%s] have joined the chatroom %s with topic %s", me, room, topic));
 
@@ -479,7 +486,8 @@ void ext_yahoo_chat_join(int id, const char *me, const char *room, const char *
 		members=n;
 	}
 }
-void ext_yahoo_chat_userjoin(int id, const char *me, const char *room, struct yahoo_chat_member *who)
+
+static void ext_yahoo_chat_userjoin(int id, const char *me, const char *room, struct yahoo_chat_member *who)
 {
 	print_chat_member(who);
 	print_message((" joined the chatroom %s [%s]", room, me));
@@ -488,11 +496,13 @@ void ext_yahoo_chat_userjoin(int id, const char *me, const char *room, struct ya
 	FREE(who->location);
 	FREE(who);
 }
-void ext_yahoo_chat_userleave(int id, const char *me, const char *room, const char *who)
+
+static void ext_yahoo_chat_userleave(int id, const char *me, const char *room, const char *who)
 {
 	print_message(("%s left the chatroom %s [%s]", who, room, me));
 }
-void ext_yahoo_chat_message(int id, const char *me, const char *who, const char *room, const char *msg, int msgtype, int utf8)
+
+static void ext_yahoo_chat_message(int id, const char *me, const char *who, const char *room, const char *msg, int msgtype, int utf8)
 {
 	char * umsg = (char *)msg;
 	char * charpos;
@@ -518,7 +528,7 @@ void ext_yahoo_chat_message(int id, const char *me, const char *who, const char
 		FREE(umsg);
 }
 
-void ext_yahoo_status_changed(int id, const char *who, int stat, const char *msg, int away, int idle, int mobile)
+static void ext_yahoo_status_changed(int id, const char *who, int stat, const char *msg, int away, int idle, int mobile)
 {
 	yahoo_account * ya=NULL;
 	YList * b;
@@ -570,7 +580,7 @@ void ext_yahoo_status_changed(int id, const char *who, int stat, const char *msg
 	}
 }
 
-void ext_yahoo_got_buddies(int id, YList * buds)
+static void ext_yahoo_got_buddies(int id, YList * buds)
 {
 	while(buddies) {
 		FREE(buddies->data);
@@ -592,11 +602,28 @@ void ext_yahoo_got_buddies(int id, YList * buds)
 	}
 }
 
-void ext_yahoo_got_ignore(int id, YList * igns)
+static void ext_yahoo_got_ignore(int id, YList * igns)
 {
 }
 
-void ext_yahoo_got_im(int id, const char *me, const char *who, const char *msg, long tm, int stat, int utf8)
+static void ext_yahoo_got_buzz(int id, const char *me, const char *who, long tm)
+{
+	who = get_buddy_name(who);
+
+	printf("\a");
+	if(tm) {
+		char timestr[255];
+
+		strncpy(timestr, ctime((time_t *)&tm), sizeof(timestr));
+		timestr[strlen(timestr) - 1] = '\0';
+
+		print_message(("[Offline message at %s to %s from %s]: **DING**",
+				timestr, me, who))
+	} else
+		print_message(("[%s]%s: **DING**", me, who))
+}
+
+static void ext_yahoo_got_im(int id, const char *me, const char *who, const char *msg, long tm, int stat, int utf8)
 {
 	char *umsg = (char *)msg;
 
@@ -621,24 +648,21 @@ void ext_yahoo_got_im(int id, const char *me, const char *who, const char *msg,
 
 		print_message(("[Offline message at %s to %s from %s]: %s", 
 				timestr, me, who, umsg))
-	} else {
-		if(!strcmp(umsg, "<ding>")) 
-			printf("\a");
+	} else
 		print_message(("[%s]%s: %s", me, who, umsg))
-	}
 
 	if(utf8)
 		FREE(umsg);
 }
 
-void ext_yahoo_rejected(int id, const char *who, const char *msg)
+static void ext_yahoo_rejected(int id, const char *who, const char *msg)
 {
 	print_message(("%s has rejected you%s%s", who, 
 				(msg?" with the message:\n":"."), 
 				(msg?msg:"")));
 }
 
-void ext_yahoo_contact_added(int id, const char *myid, const char *who, const char *msg)
+static void ext_yahoo_contact_added(int id, const char *myid, const char *who, const char *msg)
 {
 	char buff[1024];
 
@@ -659,17 +683,17 @@ void ext_yahoo_contact_added(int id, const char *myid, const char *who, const ch
 */
 }
 
-void ext_yahoo_typing_notify(int id, const char* me, const char *who, int stat)
+static void ext_yahoo_typing_notify(int id, const char* me, const char *who, int stat)
 {
 	if(stat && do_typing_notify)
 		print_message(("[%s]%s is typing...", me, who));
 }
 
-void ext_yahoo_game_notify(int id, const char *me, const char *who, int stat)
+static void ext_yahoo_game_notify(int id, const char *me, const char *who, int stat, const char *msg)
 {
 }
 
-void ext_yahoo_mail_notify(int id, const char *from, const char *subj, int cnt)
+static void ext_yahoo_mail_notify(int id, const char *from, const char *subj, int cnt)
 {
 	char buff[1024] = {0};
 	
@@ -692,7 +716,7 @@ void ext_yahoo_mail_notify(int id, const char *from, const char *subj, int cnt)
 		print_message((buff));
 }
 
-void ext_yahoo_got_webcam_image(int id, const char *who,
+static void ext_yahoo_got_webcam_image(int id, const char *who,
 		const unsigned char *image, unsigned int image_size, unsigned int real_size,
 		unsigned int timestamp)
 {
@@ -733,7 +757,7 @@ void ext_yahoo_got_webcam_image(int id, const char *who,
 	}
 }
 
-void ext_yahoo_webcam_viewer(int id, const char *who, int connect)
+static void ext_yahoo_webcam_viewer(int id, const char *who, int connect)
 {
 	switch (connect)
 	{
@@ -750,7 +774,7 @@ void ext_yahoo_webcam_viewer(int id, const char *who, int connect)
 	}
 }
 
-void ext_yahoo_webcam_closed(int id, const char *who, int reason)
+static void ext_yahoo_webcam_closed(int id, const char *who, int reason)
 {
 	switch(reason)
 	{
@@ -769,7 +793,7 @@ void ext_yahoo_webcam_closed(int id, const char *who, int reason)
 	}
 }
 
-void ext_yahoo_webcam_data_request(int id, int send)
+static void ext_yahoo_webcam_data_request(int id, int send)
 {
 	webcam_id = id;
 
@@ -783,12 +807,12 @@ void ext_yahoo_webcam_data_request(int id, int send)
 	send_webcam_images = send;
 }
 
-void ext_yahoo_webcam_invite(int id, const char *me, const char *from)
+static void ext_yahoo_webcam_invite(int id, const char *me, const char *from)
 {
 	print_message(("Got a webcam invitation to %s from %s", me, from));
 }
 
-void ext_yahoo_webcam_invite_reply(int id, const char *me, const char *from, int accept)
+static void ext_yahoo_webcam_invite_reply(int id, const char *me, const char *from, int accept)
 {
 	if(accept) {
 		print_message(("[%s]%s accepted webcam invitation...", me, from));
@@ -797,7 +821,7 @@ void ext_yahoo_webcam_invite_reply(int id, const char *me, const char *from, int
 	}
 }
 
-void ext_yahoo_system_message(int id, const char *msg)
+static void ext_yahoo_system_message(int id, const char *me, const char *who, const char *msg)
 {
 	if(ignore_system)
 		return;
@@ -843,7 +867,7 @@ void yahoo_logout()
 	print_message(("logged_out"));
 }
 
-void ext_yahoo_login(yahoo_local_account * ylad, int login_mode)
+static void ext_yahoo_login(yahoo_local_account * ylad, int login_mode)
 {
 	LOG(("ext_yahoo_login"));
 
@@ -862,12 +886,12 @@ void ext_yahoo_login(yahoo_local_account * ylad, int login_mode)
 	rearm(&pingTimer, 600);
 }
 
-void ext_yahoo_got_cookies(int id)
+static void ext_yahoo_got_cookies(int id)
 {
 	/*yahoo_get_yab(id);*/
 }
 
-void ext_yahoo_login_response(int id, int succ, const char *url)
+static void ext_yahoo_login_response(int id, int succ, const char *url)
 {
 	char buff[1024];
 
@@ -903,7 +927,7 @@ void ext_yahoo_login_response(int id, int succ, const char *url)
 	poll_loop=0;
 }
 
-void ext_yahoo_error(int id, const char *err, int fatal, int num)
+static void ext_yahoo_error(int id, const char *err, int fatal, int num)
 {
 	fprintf(stdout, "Yahoo Error: ");
 	fprintf(stdout, "%s", err);
@@ -958,54 +982,9 @@ void yahoo_set_current_state(int yahoo_state)
 		yahoo_set_away(ylad->id, yahoo_state, NULL, 1);
 }
 
-int ext_yahoo_connect(const char *host, int port)
+static int ext_yahoo_connect(const char *host, int port)
 {
-	struct sockaddr_in serv_addr;
-	static struct hostent *server;
-	static char last_host[256];
-	int servfd;
-	char **p;
-
-	if(last_host[0] || strcasecmp(last_host, host)!=0) {
-		if(!(server = gethostbyname(host))) {
-			WARNING(("failed to look up server (%s:%d)\n%d: %s", 
-						host, port,
-						h_errno, strerror(h_errno)));
-			return -1;
-		}
-		strncpy(last_host, host, 255);
-	}
-
-	if((servfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-		WARNING(("Socket create error (%d): %s", errno, strerror(errno)));
-		return -1;
-	}
-
-	LOG(("connecting to %s:%d", host, port));
-
-	for (p = server->h_addr_list; *p; p++)
-	{
-		memset(&serv_addr, 0, sizeof(serv_addr));
-		serv_addr.sin_family = AF_INET;
-		memcpy(&serv_addr.sin_addr.s_addr, *p, server->h_length);
-		serv_addr.sin_port = htons(port);
-
-		LOG(("trying %s", inet_ntoa(serv_addr.sin_addr)));
-		if(connect(servfd, (struct sockaddr *) &serv_addr, 
-					sizeof(serv_addr)) == -1) {
-			if(errno!=ECONNREFUSED && errno!=ETIMEDOUT && 
-					errno!=ENETUNREACH) {
-				break;
-			}
-		} else {
-			LOG(("connected"));
-			return servfd;
-		}
-	}
-
-	WARNING(("Could not connect to %s:%d\n%d:%s", host, port, errno, 
-				strerror(errno)));
-	close(servfd);
+	WARNING(("This should not be used anymore. File a bug report."));
 	return -1;
 }
 
@@ -1014,46 +993,78 @@ int ext_yahoo_connect(const char *host, int port)
  */
 YList *connections = NULL;
 struct _conn {
-	int tag;
 	int fd;
-	int id;
-	yahoo_input_condition cond;
-	void *data;
+	SSL *ssl;
+	int use_ssl;
 	int remove;
 };
+
+struct conn_handler {
+	struct _conn *con;
+	int id;
+	int tag;
+	yahoo_input_condition cond;
+	int remove;
+	void *data;
+};
+
 static int connection_tags=0;
 
-int ext_yahoo_add_handler(int id, int fd, yahoo_input_condition cond, void *data)
+static int ext_yahoo_add_handler(int id, void *d, yahoo_input_condition cond, void *data)
 {
-	struct _conn *c = y_new0(struct _conn, 1);
-	c->tag = ++connection_tags;
-	c->id = id;
-	c->fd = fd;
-	c->cond = cond;
-	c->data = data;
+	struct conn_handler *h = y_new0(struct conn_handler, 1);
 
-	LOG(("Add %d for %d, tag %d", fd, id, c->tag));
+	h->id = id;
+	h->tag = ++connection_tags;
+	h->con = d;
+	h->cond = cond;
+	h->data = data;
 
-	connections = y_list_prepend(connections, c);
+	LOG(("Add %d(%d) for %d, tag %d", h->con->fd, cond, id, h->tag));
 
-	return c->tag;
+	connections = y_list_prepend(connections, h);
+
+	return h->tag;
 }
 
-void ext_yahoo_remove_handler(int id, int tag)
+static void ext_yahoo_remove_handler(int id, int tag)
 {
 	YList *l;
+	if (!tag)
+		return;
+
 	for(l = connections; l; l = y_list_next(l)) {
-		struct _conn *c = l->data;
+		struct conn_handler *c = l->data;
 		if(c->tag == tag) {
 			/* don't actually remove it, just mark it for removal */
 			/* we'll remove when we start the next poll cycle */
-			LOG(("Marking id:%d fd:%d tag:%d for removal", c->id, c->fd, c->tag));
+			LOG(("Marking id:%d fd:%p tag:%d for removal",
+				c->id, c->con, c->tag));
 			c->remove = 1;
 			return;
 		}
 	}
 }
 
+static SSL *do_ssl_connect(int fd)
+{
+	SSL *ssl;
+	SSL_CTX *ctx;
+
+	LOG(("SSL Handshake"));
+
+	SSL_library_init ();
+	ctx = SSL_CTX_new(SSLv23_client_method());
+	ssl = SSL_new(ctx);
+	SSL_CTX_free(ctx);
+	SSL_set_fd(ssl, fd);
+
+	if (SSL_connect(ssl) == 1)
+		return ssl;
+
+	return NULL;
+}
+
 struct connect_callback_data {
 	yahoo_connect_callback callback;
 	void * callback_data;
@@ -1061,37 +1072,51 @@ struct connect_callback_data {
 	int tag;
 };
 
-static void connect_complete(void *data, int source, yahoo_input_condition condition)
+static void connect_complete(void *data, struct _conn *source, yahoo_input_condition condition)
 {
 	struct connect_callback_data *ccd = data;
 	int error, err_size = sizeof(error);
 
 	ext_yahoo_remove_handler(0, ccd->tag);
-	getsockopt(source, SOL_SOCKET, SO_ERROR, &error, (socklen_t *)&err_size);
+	getsockopt(source->fd, SOL_SOCKET, SO_ERROR, &error, (socklen_t *)&err_size);
 
-	if(error) {
-		close(source);
-		source = -1;
+	if(error)
+		goto err;
+
+	LOG(("Connected fd: %d, error: %d", source->fd, error));
+
+	if (source->use_ssl) {
+		source->ssl = do_ssl_connect(source->fd);
+
+		if (!source->ssl) {
+err:
+			LOG(("SSL Handshake Failed!"));
+			ext_yahoo_close(source);
+
+			ccd->callback(NULL, 0, ccd->callback_data);
+			FREE(ccd);
+			return;
+		}
 	}
 
-	LOG(("Connected fd: %d, error: %d", source, error));
+	fcntl(source->fd, F_SETFL, O_NONBLOCK);
 
 	ccd->callback(source, error, ccd->callback_data);
 	FREE(ccd);
 }
 
-void yahoo_callback(struct _conn *c, yahoo_input_condition cond)
+void yahoo_callback(struct conn_handler *c, yahoo_input_condition cond)
 {
 	int ret=1;
 	char buff[1024]={0};
 
 	if(c->id < 0) {
-		connect_complete(c->data, c->fd, cond);
+		connect_complete(c->data, c->con, cond);
 	} else {
 		if(cond & YAHOO_INPUT_READ)
-			ret = yahoo_read_ready(c->id, c->fd, c->data);
+			ret = yahoo_read_ready(c->id, c->con, c->data);
 		if(ret>0 && cond & YAHOO_INPUT_WRITE)
-			ret = yahoo_write_ready(c->id, c->fd, c->data);
+			ret = yahoo_write_ready(c->id, c->con, c->data);
 
 		if(ret == -1)
 			snprintf(buff, sizeof(buff), 
@@ -1105,15 +1130,62 @@ void yahoo_callback(struct _conn *c, yahoo_input_condition cond)
 	}
 }
 
-int ext_yahoo_connect_async(int id, const char *host, int port, 
-		yahoo_connect_callback callback, void *data)
+static int ext_yahoo_write(void *fd, char *buf, int len)
+{
+	struct _conn *c = fd;
+
+	if (c->use_ssl)
+		return SSL_write(c->ssl, buf, len);
+	else
+		return write(c->fd, buf, len);
+}
+
+static int ext_yahoo_read(void *fd, char *buf, int len)
+{
+	struct _conn *c = fd;
+
+	if (c->use_ssl)
+		return SSL_read(c->ssl, buf, len);
+	else
+		return read(c->fd, buf, len);
+}
+
+static void ext_yahoo_close(void *fd)
+{
+	struct _conn *c = fd;
+	YList *l;
+
+	if (c->use_ssl)
+		SSL_free(c->ssl);
+
+	close(c->fd);
+	c->fd = 0;
+
+	/* Remove all handlers */
+	for (l = connections; l; l = y_list_next(l)) {
+		struct conn_handler *h = l->data;
+
+		if (h->con == c)
+			h->remove = 1;
+	}
+
+	c->remove = 1;
+}
+
+static int ext_yahoo_connect_async(int id, const char *host, int port, 
+		yahoo_connect_callback callback, void *data, int use_ssl)
 {
 	struct sockaddr_in serv_addr;
 	static struct hostent *server;
 	int servfd;
 	struct connect_callback_data * ccd;
 	int error;
+	SSL *ssl = NULL;
 
+	struct _conn *c;
+
+	LOG(("Connecting to %s", host));
+	
 	if(!(server = gethostbyname(host))) {
 		errno=h_errno;
 		return -1;
@@ -1128,11 +1200,31 @@ int ext_yahoo_connect_async(int id, const char *host, int port,
 	memcpy(&serv_addr.sin_addr.s_addr, *server->h_addr_list, server->h_length);
 	serv_addr.sin_port = htons(port);
 
+	c = y_new0(struct _conn, 1);
+	c->fd = servfd;
+	c->use_ssl = use_ssl;
+
 	error = connect(servfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr));
 
 	LOG(("Trying to connect: fd:%d error:%d", servfd, error));
 	if(!error) {
-		callback(servfd, 0, data);
+		LOG(("Connected"));
+		if (use_ssl) {
+			ssl = do_ssl_connect(servfd);
+
+			if (!ssl) {
+				LOG(("SSL Handshake Failed!"));
+				ext_yahoo_close(c);
+
+				callback(NULL, 0, data);
+				return -1;
+			}
+		}
+
+		c->ssl = ssl;
+		fcntl(c->fd, F_SETFL, O_NONBLOCK);
+
+		callback(c, 0, data);
 		return 0;
 	} else if(error == -1 && errno == EINPROGRESS) {
 		ccd = calloc(1, sizeof(struct connect_callback_data));
@@ -1140,13 +1232,15 @@ int ext_yahoo_connect_async(int id, const char *host, int port,
 		ccd->callback_data = data;
 		ccd->id = id;
 
-		ccd->tag = ext_yahoo_add_handler(-1, servfd, YAHOO_INPUT_WRITE, ccd);
+		ccd->tag = ext_yahoo_add_handler(-1, c, YAHOO_INPUT_WRITE, ccd);
 		return ccd->tag;
 	} else {
-		if(error == -1) {
-			LOG(("%s", strerror(errno)));
-		}
-		close(servfd);
+		if(error == -1)
+			LOG(("Connection failure: %s", strerror(errno)));
+
+		ext_yahoo_close(c);
+
+		callback(NULL, 0, data);
 		return -1;
 	}
 }
@@ -1160,6 +1254,7 @@ static void process_commands(char *line)
 
 	char *tmp, *start;
 	char *copy = strdup(line);
+	int yid = 0;
 
 	enum yahoo_status state;
 
@@ -1431,6 +1526,13 @@ static void process_commands(char *line)
 		struct yab * yab;
 		
 		to = copy;
+		tmp = strchr(copy, ' ');
+		if(tmp) {
+			*tmp = '\0';
+			copy = tmp+1;
+		}
+		yid = atoi(copy);
+
 		tmp = strchr(copy, ' ');
 		if(tmp) {
 			*tmp = '\0';
@@ -1441,7 +1543,8 @@ static void process_commands(char *line)
 		if(to && msg) {
 			yab = y_new0(struct yab, 1);
 			yab->id = to;
-			yab->fname = msg;
+			yab->yid = yid;	/* Only do this if you have got it from the server */
+			yab->nname = msg;
 			yahoo_set_yab(ylad->id, yab);
 			FREE(yab);
 		}
@@ -1599,21 +1702,21 @@ int main(int argc, char * argv[])
 		lfd=0;
 
 		for(l=connections; l; ) {
-			struct _conn *c = l->data;
+			struct conn_handler *c = l->data;
 			if(c->remove) {
 				YList *n = y_list_next(l);
-				LOG(("Removing id:%d fd:%d", c->id, c->fd));
+				LOG(("Removing id:%d fd:%d", c->id, c->con->fd));
 				connections = y_list_remove_link(connections, l);
 				y_list_free_1(l);
-				free(c);
+				FREE(c);
 				l=n;
 			} else {
 				if(c->cond & YAHOO_INPUT_READ)
-					FD_SET(c->fd, &inp);
+					FD_SET(c->con->fd, &inp);
 				if(c->cond & YAHOO_INPUT_WRITE)
-					FD_SET(c->fd, &outp);
-				if(lfd < c->fd)
-					lfd = c->fd;
+					FD_SET(c->con->fd, &outp);
+				if(lfd < c->con->fd)
+					lfd = c->con->fd;
 				l = y_list_next(l);
 			}
 		}
@@ -1628,12 +1731,17 @@ int main(int argc, char * argv[])
 #endif
 
 		for(l = connections; l; l = y_list_next(l)) {
-			struct _conn *c = l->data;
+			struct conn_handler *c = l->data;
+			if(c->con->remove) {
+				FREE(c->con);
+				c->con = NULL;
+				continue;
+			}
 			if(c->remove)
 				continue;
-			if(FD_ISSET(c->fd, &inp))
+			if(FD_ISSET(c->con->fd, &inp))
 				yahoo_callback(c, YAHOO_INPUT_READ);
-			if(FD_ISSET(c->fd, &outp))
+			if(FD_ISSET(c->con->fd, &outp))
 				yahoo_callback(c, YAHOO_INPUT_WRITE);
 		}
 
@@ -1644,8 +1752,7 @@ int main(int argc, char * argv[])
 
 	while(connections) {
 		YList *tmp = connections;
-		struct _conn * c = connections->data;
-		close(c->fd);
+		struct conn_handler *c = connections->data;
 		FREE(c);
 		connections = y_list_remove_link(connections, connections);
 		y_list_free_1(tmp);
@@ -1661,55 +1768,76 @@ int main(int argc, char * argv[])
 	return 0;
 }
 
-void ext_yahoo_got_file(int id, const char *me, const char *who, const char *url, long expires, const char *msg, const char *fname, unsigned long fesize)
+static void ext_yahoo_got_file(int id, const char *me, const char *who, const char *msg, const char *fname, 
+	unsigned long fesize, char *trid)
+{
+	LOG(("Got a File transfer request (%s, %ld bytes) from %s", fname, fesize, who));
+}
+
+static void ext_yahoo_file_transfer_done(int id, int response, void *data)
 {
 }
 
-void ext_yahoo_got_identities(int id, YList * ids)
+static char *ext_yahoo_get_ip_addr(const char *domain)
+{
+	return NULL;
+}
+
+static void ext_yahoo_got_ft_data(int id, const unsigned char *in, int count, void *data)
 {
 }
-void ext_yahoo_chat_yahoologout(int id, const char *me)
+
+static void ext_yahoo_got_identities(int id, YList * ids)
+{
+}
+
+static void ext_yahoo_chat_yahoologout(int id, const char *me)
 { 
  	LOG(("got chat logout for %s", me));
 }
-void ext_yahoo_chat_yahooerror(int id, const char *me)
+
+static void ext_yahoo_chat_yahooerror(int id, const char *me)
 { 
  	LOG(("got chat error for %s", me));
 }
 
-void ext_yahoo_got_ping(int id, const char *errormsg)
+static void ext_yahoo_got_ping(int id, const char *errormsg)
 { 
  	LOG(("got ping errormsg %s", errormsg));
 }
 
-void ext_yahoo_got_search_result(int id, int found, int start, int total, YList *contacts)
+static void ext_yahoo_got_search_result(int id, int found, int start, int total, YList *contacts)
 {
 	LOG(("got search result"));
 }
 
-void ext_yahoo_got_buddyicon_checksum(int id, const char *a, const char *b, int checksum)
+static void ext_yahoo_got_buddyicon_checksum(int id, const char *a, const char *b, int checksum)
 {
 	LOG(("got buddy icon checksum"));
 }
 
-void ext_yahoo_got_buddyicon(int id, const char *a, const char *b, const char *c, int checksum)
+static void ext_yahoo_got_buddy_change_group(int id, const char *me, const char *who, 
+	const char *old_group, const char *new_group)
+{
+}
+
+static void ext_yahoo_got_buddyicon(int id, const char *a, const char *b, const char *c, int checksum)
 {
 	LOG(("got buddy icon"));
 }
 
-void ext_yahoo_buddyicon_uploaded(int id, const char *url)
+static void ext_yahoo_buddyicon_uploaded(int id, const char *url)
 {
 	LOG(("buddy icon uploaded"));
 }
 
-void ext_yahoo_got_buddyicon_request(int id, const char *me, const char *who)
+static void ext_yahoo_got_buddyicon_request(int id, const char *me, const char *who)
 {
 	LOG(("got buddy icon request from %s",who));
 }
 
 static void register_callbacks()
 {
-#ifdef USE_STRUCT_CALLBACKS
 	static struct yahoo_callbacks yc;
 
 	yc.ext_yahoo_login_response = ext_yahoo_login_response;
@@ -1719,6 +1847,7 @@ static void register_callbacks()
 	yc.ext_yahoo_got_cookies = ext_yahoo_got_cookies;
 	yc.ext_yahoo_status_changed = ext_yahoo_status_changed;
 	yc.ext_yahoo_got_im = ext_yahoo_got_im;
+	yc.ext_yahoo_got_buzz = ext_yahoo_got_buzz;
 	yc.ext_yahoo_got_conf_invite = ext_yahoo_got_conf_invite;
 	yc.ext_yahoo_conf_userdecline = ext_yahoo_conf_userdecline;
 	yc.ext_yahoo_conf_userjoin = ext_yahoo_conf_userjoin;
@@ -1738,6 +1867,9 @@ static void register_callbacks()
 	yc.ext_yahoo_webcam_viewer = ext_yahoo_webcam_viewer;
 	yc.ext_yahoo_webcam_data_request = ext_yahoo_webcam_data_request;
 	yc.ext_yahoo_got_file = ext_yahoo_got_file;
+	yc.ext_yahoo_got_ft_data = ext_yahoo_got_ft_data;
+	yc.ext_yahoo_get_ip_addr = ext_yahoo_get_ip_addr;
+	yc.ext_yahoo_file_transfer_done = ext_yahoo_file_transfer_done;
 	yc.ext_yahoo_contact_added = ext_yahoo_contact_added;
 	yc.ext_yahoo_rejected = ext_yahoo_rejected;
 	yc.ext_yahoo_typing_notify = ext_yahoo_typing_notify;
@@ -1751,14 +1883,16 @@ static void register_callbacks()
 	yc.ext_yahoo_remove_handler = ext_yahoo_remove_handler;
 	yc.ext_yahoo_connect = ext_yahoo_connect;
 	yc.ext_yahoo_connect_async = ext_yahoo_connect_async;
+	yc.ext_yahoo_read = ext_yahoo_read;
+	yc.ext_yahoo_write = ext_yahoo_write;
+	yc.ext_yahoo_close = ext_yahoo_close;
 	yc.ext_yahoo_got_buddyicon = ext_yahoo_got_buddyicon;
 	yc.ext_yahoo_got_buddyicon_checksum = ext_yahoo_got_buddyicon_checksum;
 	yc.ext_yahoo_buddyicon_uploaded = ext_yahoo_buddyicon_uploaded;
 	yc.ext_yahoo_got_buddyicon_request = ext_yahoo_got_buddyicon_request;
 	yc.ext_yahoo_got_ping = ext_yahoo_got_ping;
+	yc.ext_yahoo_got_buddy_change_group = ext_yahoo_got_buddy_change_group;
 
 	yahoo_register_callbacks(&yc);
-	
-#endif
 }
 
